'use strict'

const argx = require('argx')
const apemanfile = require('apemanfile');
const apemancwd = require('apemancwd');
const arrayreduce = require('arrayreduce');
const arrayfilter = require('arrayfilter');
const apemandocker = require('apemandocker');
const co = require('co');
const runDply = require('./deploying/run_dply');
const listDply = require('./deploying/list_dply');
const ttyDply = require('./deploying/tty_dply');
const logDply = require('./deploying/log_dply');
const removeDply = require('./deploying/remove_dply');
const execCmd = require('./deploying/exec_cmd');
const nameDply = require('./deploying/name_dply');

let concatArray = arrayreduce.arrayConcat()
let rejectEmpty = arrayfilter.emptyReject();

/** @lends apemanDply */
function apemanDply (names, options) {
  let args = argx(arguments)
  options = args.pop('object') || {}
  names = args.remain().filter(rejectEmpty).reduce(concatArray, [])

  let docker = apemandocker.create()
  let cwd = apemancwd.create({ prefix: 'apeman-dply' })

  let configuration = apemanfile(options.configuration)
  return co(function * () {
    cwd.chdir(configuration.$cwd)

    let duplicate = [
        options.list,
        options.delete,
        options.rerun,
        options.tty,
        options.exec,
        options.print,
        options.printon
      ].filter((command) => !!command
      ).length > 1
    if (duplicate) {
      throw new Error(`Invalid option: ${JSON.stringify(options, null, 2)}`)
    }
    if (options.rerun) {
      yield removeDply(configuration, names, {
        force: options.force
      })
      return yield runDply(configuration, names, {
        force: options.force
      })
    }

    if (options.exec) {
      return yield execCmd(configuration, names, options.exec, {})

    }

    if (options.list) {
      return yield listDply(configuration, names, {})
    }

    if (options.delete) {
      return yield removeDply(configuration, names, {
        force: options.force
      })
    }

    if (options.tty) {
      return yield ttyDply(configuration, names, {})
    }

    if (options.print) {
      return yield logDply(configuration, names, {})
    }

    if (options.printon) {
      return yield logDply(configuration, names, {
        follow: true
      })
    }

    let result = yield runDply(configuration, names, {
      force: options.force
    })
    cwd.restoreAll()
    return result
  })
}

apemanDply.nameDply = nameDply

module.exports = apemanDply
