/**
 * @memberof module:apeman-dply
 * @function getDplys
 * @param {Apemanfile} apemanfile
 * @param {string[]} names
 */

'use strict'

const arrayreduce = require('arrayreduce')
const arrayfilter = require('arrayfilter')
const stringcase = require('stringcase')
const minimatch = require('minimatch')
const apemandocker = require('apemandocker')
const Dply = require('./dply')
const nameDply = require('./name_dply')
const findout = require('findout')

let rejectEmpty = arrayfilter.emptyReject()
let concatArray = arrayreduce.arrayConcat()

/** @lends getDplys */
function getDplys (apemanfile, names) {
  let $pkg = apemanfile.$pkg;
  if (!$pkg) {
    throw new Error(`$pkg not found.`)
  }

  let $dplys = apemanfile.$dplys || {}

  return Object.keys($dplys)
    .filter((filtering) => {
      let hasFilter = names && names.length > 0;
      if (!hasFilter) {
        return true
      }
      for (let i = 0, len = names.length; i < len; i++) {
        let hit = minimatch(filtering, names[ i ])
        if (hit) {
          return true
        }
      }
      return false
    })
    .map((name) => {
      let dply = $dplys[ name ];
      return [].concat(dply || [])
        .reduce(concatArray, [])
        .filter(rejectEmpty)
        .map((conf) => {
          if (typeof(conf) === 'string') {
            return findout(conf)
          }
          return conf;
        })
        .map((conf, i) => {
          if (conf.infra) {
            console.warn('conf.infra is deprecated. use conf.infr instead.')
            conf.infr = conf.infra;
          }
          if (!conf.infr) {
            conf.infr = name;
          }
          return new Dply({
            name: name,
            hostname: conf.hostname,
            container: nameDply(name, $pkg),
            port: conf.port,
            mount: conf.mount,
            env: conf.env,
            index: [ i ].join('-'),
            image: apemandocker.nameImage($pkg, conf.infr),
            workdir: conf.workdir
          })
        })
    })
    .sort((a, b) =>
      String(a.name).localeCompare(String(b.name))
    )
    .reduce(concatArray, [])
}

module.exports = getDplys
