/**
 * @function removeDply
 */
'use strict'

const co = require('co');
const apemandocker = require('apemandocker');
const getDplys = require('./get_dplys');
const RemoveLogger = require('../logging/remove_logger');

/** @lends removeDply */
function removeDply (apemanfile, names, options = {}) {
  let docker = apemandocker.create({})
  let logger = new RemoveLogger({})
  let dplys = getDplys(apemanfile, names)

  let networkName = apemandocker.nameNetwork(apemanfile.$pkg)

  return co(function * () {
    for (let dply of [].concat(dplys)) {
      yield removeDply.one(docker, logger, dply, options)
    }
    if (names.length === 0) {
      yield removeDply.network(docker, logger, networkName, options)
    }
  })
}

removeDply.network = function network (docker, logger, name, options = {}) {
  return co(function * () {
    let exists = yield docker.networkExists(name)
    if (exists) {
      yield docker.removeNetwork(name, {})
    }
    logger.logNetworkRemoved(name)
  })
}

removeDply.one = function one (docker, logger, dply, options) {
  let { name, container } = dply
  return co(function * () {
    logger.logRemoveStarted(name)

    let exists = yield docker.containerExists(container)
    if (exists) {
      yield docker.removeContainer(container, {
        force: !!options.force
      })
    }
    logger.logRemoveDone(name)
  })
}

module.exports = removeDply;
