/**
 * @function runDply
 */
'use strict'

const path = require('path')
const co = require('co')
const apemandocker = require('apemandocker')
const getDplys = require('./get_dplys')
const RunLogger = require('../logging/run_logger')

/** @lends runDply */
function runDply (apemanfile, names, options = {}) {
  let docker = apemandocker.create({})
  let logger = new RunLogger({});
  let dplys = getDplys(apemanfile, names);

  let networkName = apemandocker.nameNetwork(apemanfile.$pkg)
  return co(function * () {
    yield runDply.network(docker, logger, networkName, {})
    for (let dply of [].concat(dplys)) {
      yield runDply.one(docker, logger, dply, {
        networkName: networkName
      })
    }
  })
}

runDply.network = function (docker, logger, name, options) {
  return co(function * () {
    let exists = yield docker.networkExists(name)
    if (exists) {
      return;
    }
    yield docker.createNetwork(name, {})
    logger.logNetworkCreated(name)
  })
}

runDply.one = function (docker, logger, dply, options) {
  let container = dply.container,
    name = dply.name,
    image = dply.image;

  let mount = dply.mount,
    port = dply.port,
    env = dply.env,
    hostname = dply.hostname,
    workdir = dply.workdir || null;

  let networkName = options.networkName;

  return co(function * () {
    logger.logRunStarted(name, dply)
    let exists = yield docker.containerExists(container)
    if (exists) {
      let running = yield docker.isContainerRunning(container)
      if (running) {
        logger.warnAlreadyRunning(container)
        return;
      }
      yield docker.startContainer(container, {
        attach: false
      })
    } else {
      yield docker.runContainer(image, {
        name: container,
        detach: true,
        hostname: hostname,
        net: options.net,
        publish: port && Object.keys(port).map((key) => {
          return [ port[ key ], key ].join(':')
        }),
        volume: mount && Object.keys(mount).map((key) => {
          return [ path.resolve(mount[ key ]), key ].join(':')
        }),
        env: env && Object.keys(env).map((key) => {
          return [ key, env[ key ] ].join('=')
        }),
        workdir: workdir
      })
      yield  docker.connectNetwork(networkName, container, {})
    }
    logger.logRunDone(name)
  })
}

module.exports = runDply
