/**
 * Logger for run.
 * @constructor RunLogger
 */

'use strict'

const colorprint = require('colorprint')
const path = require('path')
const arrayfilter = require('arrayfilter')
const textconv = require('textconv')

/** @lends RunLogger */
module.exports = colorprint.define(
  /** @lends RunLogger.prototype */
  {
    logNetworkCreated (name) {
      const s = this
      s.info(`Network created: ${name}`)
    },
    logRunStarted (name, dply) {
      const s = this
      s.info(`Deploying ${name}...`)

      s.debug('')
      s._logValue('Name', dply.name)
      s._logValue('Image', dply.image)
      s._logValue('Container', dply.container)
      s._logValue('Infr', dply.infr)
      s._logValues('Mount', dply.mount, ' -> ', true)
      s._logValue('Hostname', dply.hostname)
      s._logValues('Publish', dply.port, ' -> ', true)
      s._logValues('Env', dply.env, ': ')
      s._logValue('Workdir', dply.workdir)
      s.debug('')
    },
    logRunDone (name) {
      const s = this
      s.info(`...deploying ${name} done!`)
      s.debug('')
    },
    warnAlreadyRunning (name) {
      const s = this
      s.warn(`Container already running: ${name}`)
    },
    _logValue (label, val) {
      if (!val) {
        return;
      }
      const s = this
      s.debug(`  ${label}:`)
      s.debug(`    ${val}`)
      s.debug('')
    },
    _logValues (label, data, joiner, reverse) {
      if (!data) {
        return
      }
      const s = this
      s.debug(`  ${label}:`)
      Object.keys(data).forEach((key) => {
        let val = data[ key ]
        if (reverse) {
          s.debug(`    ${val}${joiner}${key}`)
        } else {
          s.debug(`    ${key}${joiner}${val}`)

        }
      })
      s.debug('')
    }
  }
)
