/**
 * Test case for apemanDply.
 * Runs with mocha.
 */
'use strict'

const apemanDply = require('../lib/apeman_dply.js')
const apemanInfr = require('apeman-infr')
const injectmock = require('injectmock')
const assert = require('assert')

describe('apeman-dply', function () {
  this.timeout(32000)

  let Apemanfile = require.resolve('../doc/mocks/mock-project/Apemanfile')

  before((done) => {
    done()
  })

  after((done) => {
    injectmock.restoreAll()
    done()
  })

  it('Build a infra', (done) => {
    apemanInfr({
      configuration: Apemanfile
    }).then(
      result => done(),
      err => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('List deploy', (done) => {
    apemanDply({
      configuration: Apemanfile,
      list: true
    }).then(
      result => done(),
      err => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Do deploy', (done) => {
    apemanDply({
      configuration: Apemanfile
    }).then(
      result => done(),
      err => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Re deploy', (done) => {
    apemanDply({
      rerun: true,
      force: true,
      configuration: Apemanfile
    }).then(
      result => done(),
      err => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Exec command ', (done) => {
    apemanDply({
      exec: "ls -l",
      configuration: Apemanfile
    }).then(
      result => done(),
      err => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Open tty', (done) => {
    apemanDply("hoge", {
      tty: true,
      configuration: Apemanfile
    }).then(
      terminal => {
        done()
      }).catch(err => {
      assert.ok(!!err)
      done()
    })
  })

  it('Remove deploy', (done) => {
    apemanDply({
      configuration: Apemanfile,
      delete: true,
      force: true
    }).then(
      result => done(),
      err => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Remove a infra', (done) => {
    apemanInfr({
      delete: true,
      configuration: Apemanfile,
      force: true
    }).then(
      result => done(),
      err => {
        assert.ifError(err)
        done()
      }
    )
  })

  it('Try invalid options', (done) => {
    apemanDply({
      rerun: true,
      delete: true,
      tty: true,
      force: true
    }).then(
      result => {
        throw new Error('Invalid result')
      },
      err => {
        assert.ok(!!err)
        done()
      }
    )
  })

})

/* global describe, before, after, it */
